package sun.awt;

import java.util.HashMap;
import java.security.PrivilegedAction;
import java.util.Map;

public class OSInfo
{
    public static final OSInfo.WindowsVersion WINDOWS_UNKNOWN;
    public static final OSInfo.WindowsVersion WINDOWS_95;
    public static final OSInfo.WindowsVersion WINDOWS_98;
    public static final OSInfo.WindowsVersion WINDOWS_ME;
    public static final OSInfo.WindowsVersion WINDOWS_2000;
    public static final OSInfo.WindowsVersion WINDOWS_XP;
    public static final OSInfo.WindowsVersion WINDOWS_2003;
    public static final OSInfo.WindowsVersion WINDOWS_VISTA;
    private static final String OS_NAME = "os.name";
    private static final String OS_VERSION = "os.version";
    private static final Map<String, OSInfo.WindowsVersion> windowsVersionMap;
    private static final PrivilegedAction<OSInfo.OSType> osTypeAction;
    
    private OSInfo() {
    }
    
    public static OSInfo.OSType getOSType() throws SecurityException {
        String property = System.getProperty("os.name");
        if (property != null) {
            if (property.contains("Windows")) {
                return OSInfo.OSType.WINDOWS;
            }
            if (property.contains("Linux")) {
                return OSInfo.OSType.LINUX;
            }
            if (property.contains("Solaris") || property.contains("SunOS")) {
                return OSInfo.OSType.SOLARIS;
            }
            if (property.contains("OS X")) {
                return OSInfo.OSType.MACOSX;
            }
        }
        return OSInfo.OSType.UNKNOWN;
    }
    
    public static PrivilegedAction<OSInfo.OSType> getOSTypeAction() {
        return OSInfo.osTypeAction;
    }
    
    public static OSInfo.WindowsVersion getWindowsVersion() throws SecurityException {
        String property = System.getProperty("os.version");
        if (property == null) {
            return OSInfo.WINDOWS_UNKNOWN;
        }
        synchronized (OSInfo.windowsVersionMap) {
            OSInfo.WindowsVersion windowsVersion = (OSInfo.WindowsVersion)OSInfo.windowsVersionMap.get(property);
            if (windowsVersion == null) {
                String[] split = property.split("\\.");
                if (split.length == 2) {
                    try {
                        windowsVersion = new OSInfo.WindowsVersion(Integer.parseInt(split[0]), Integer.parseInt(split[1]));
                    }
                    catch (NumberFormatException ex) {
                        return OSInfo.WINDOWS_UNKNOWN;
                    }
                    OSInfo.windowsVersionMap.put(property, windowsVersion);
                }
                else {
                    return OSInfo.WINDOWS_UNKNOWN;
                }
            }
            return windowsVersion;
        }
    }
    
    static {
        WINDOWS_UNKNOWN = new OSInfo.WindowsVersion(-1, -1);
        WINDOWS_95 = new OSInfo.WindowsVersion(4, 0);
        WINDOWS_98 = new OSInfo.WindowsVersion(4, 10);
        WINDOWS_ME = new OSInfo.WindowsVersion(4, 90);
        WINDOWS_2000 = new OSInfo.WindowsVersion(5, 0);
        WINDOWS_XP = new OSInfo.WindowsVersion(5, 1);
        WINDOWS_2003 = new OSInfo.WindowsVersion(5, 2);
        WINDOWS_VISTA = new OSInfo.WindowsVersion(6, 0);
        (windowsVersionMap = new HashMap()).put(OSInfo.WINDOWS_95.toString(), OSInfo.WINDOWS_95);
        OSInfo.windowsVersionMap.put(OSInfo.WINDOWS_98.toString(), OSInfo.WINDOWS_98);
        OSInfo.windowsVersionMap.put(OSInfo.WINDOWS_ME.toString(), OSInfo.WINDOWS_ME);
        OSInfo.windowsVersionMap.put(OSInfo.WINDOWS_2000.toString(), OSInfo.WINDOWS_2000);
        OSInfo.windowsVersionMap.put(OSInfo.WINDOWS_XP.toString(), OSInfo.WINDOWS_XP);
        OSInfo.windowsVersionMap.put(OSInfo.WINDOWS_2003.toString(), OSInfo.WINDOWS_2003);
        OSInfo.windowsVersionMap.put(OSInfo.WINDOWS_VISTA.toString(), OSInfo.WINDOWS_VISTA);
        osTypeAction = new PrivilegedAction<OSInfo.OSType>() {
            @Override
            public OSInfo.OSType run() {
                return OSInfo.getOSType();
            }
        };
    }
    
    public enum OSType
    {
        WINDOWS, 
        LINUX, 
        SOLARIS, 
        MACOSX, 
        UNKNOWN;
    }
    
    public static class WindowsVersion implements Comparable<OSInfo.WindowsVersion>
    {
        private final int major;
        private final int minor;
        
        private WindowsVersion(int major, int minor) {
            this.major = major;
            this.minor = minor;
        }
        
        public int getMajor() {
            return this.major;
        }
        
        public int getMinor() {
            return this.minor;
        }
        
        @Override
        public int compareTo(OSInfo.WindowsVersion windowsVersion) {
            int n = this.major - windowsVersion.getMajor();
            if (n == 0) {
                n = this.minor - windowsVersion.getMinor();
            }
            return n;
        }
        
        @Override
        public boolean equals(Object o) {
            return o instanceof OSInfo.WindowsVersion && this.compareTo((OSInfo.WindowsVersion)o) == 0;
        }
        
        @Override
        public int hashCode() {
            return 31 * this.major + this.minor;
        }
        
        @Override
        public String toString() {
            return this.major + "." + this.minor;
        }
    }
}
